////////////////////////////////////////////////////////////////////////////////
// i2c.c
//
// 2014/05/23 0.0 Takashi Matsuoka
//

#include <jendefs.h>				// JN516x
#include <AppHardwareApi.h>			// JN516x

#include "i2c.h"

PUBLIC bool i2c_init(enum I2C_SPEED speed)
{
	int speed_khz;

	switch (speed)
	{
	case I2C_SPEED_400KHZ:
		speed_khz = 400;
		break;
	case I2C_SPEED_100KHZ:
		speed_khz = 100;
		break;
	default:
		return FALSE;
	}

	vAHI_SiMasterConfigure(TRUE, FALSE, 16000 / 5 / speed_khz - 1);

	return TRUE;
}

PUBLIC void i2c_wait_until_finish()
{
	while (bAHI_SiMasterPollTransferInProgress())
	{
		// Nothing.
	}
}

PUBLIC void i2c_start(uint8 address, enum I2C_DIRECTION data_direction)
{
	i2c_wait_until_finish();

	vAHI_SiMasterWriteSlaveAddr(address, data_direction == I2C_DIRECTION_WRITE ? FALSE : TRUE);
	bAHI_SiMasterSetCmdReg(E_AHI_SI_START_BIT, E_AHI_SI_NO_STOP_BIT, E_AHI_SI_NO_SLAVE_READ, E_AHI_SI_SLAVE_WRITE, E_AHI_SI_SEND_ACK, E_AHI_SI_NO_IRQ_ACK);
}

PUBLIC void i2c_write(uint8 data)
{
	i2c_wait_until_finish();

	vAHI_SiMasterWriteData8(data);
	bAHI_SiMasterSetCmdReg(E_AHI_SI_NO_START_BIT, E_AHI_SI_NO_STOP_BIT, E_AHI_SI_NO_SLAVE_READ, E_AHI_SI_SLAVE_WRITE, E_AHI_SI_SEND_ACK, E_AHI_SI_NO_IRQ_ACK);
}

PUBLIC uint8 i2c_read(enum I2C_ACKNOWLEDGE acknowledge)
{
	bAHI_SiMasterSetCmdReg(E_AHI_SI_NO_START_BIT, E_AHI_SI_NO_STOP_BIT, E_AHI_SI_SLAVE_READ, E_AHI_SI_NO_SLAVE_WRITE, acknowledge == I2C_ACK ? E_AHI_SI_SEND_ACK : E_AHI_SI_SEND_NACK, E_AHI_SI_NO_IRQ_ACK);
	i2c_wait_until_finish();

	return u8AHI_SiMasterReadData8();
}

PUBLIC void i2c_stop()
{
	i2c_wait_until_finish();

	bAHI_SiMasterSetCmdReg(E_AHI_SI_NO_START_BIT, E_AHI_SI_STOP_BIT, E_AHI_SI_NO_SLAVE_READ, E_AHI_SI_NO_SLAVE_WRITE, E_AHI_SI_SEND_ACK, E_AHI_SI_NO_IRQ_ACK);
}

////////////////////////////////////////////////////////////////////////////////
